from cpl_core.logging import LoggerABC

from gismo_data.abc.migration_abc import MigrationABC
from gismo_data.db_context import DBContext


class InitialMigration(MigrationABC):

    def __init__(self, logger: LoggerABC, db: DBContext):
        self._logger = logger
        self._db = db
        self._cursor = db.cursor

    def upgrade(self):
        self._logger.debug(__name__, 'Running upgrade')

        self._cursor.execute(
            str(f"""
            CREATE TABLE IF NOT EXISTS `MigrationHistory` (
                `MigrationId` VARCHAR(255),
                `CreatedAt` DATETIME(6),
                `LastModifiedAt` DATETIME(6),
                PRIMARY KEY(`MigrationId`)
            );
            """)
        )

        self._cursor.execute(
            str(f"""
            CREATE TABLE IF NOT EXISTS `Servers` (
                `ServerId` BIGINT NOT NULL AUTO_INCREMENT,
                `DiscordServerId` BIGINT NOT NULL,
                `CreatedAt` DATETIME(6),
                `LastModifiedAt` DATETIME(6),
                PRIMARY KEY(`ServerId`)
            );
            """)
        )

        self._cursor.execute(
            str(f"""
            CREATE TABLE IF NOT EXISTS `Users` (
                `UserId` BIGINT NOT NULL AUTO_INCREMENT,
                `DiscordId` BIGINT NOT NULL,
                `XP` BIGINT NOT NULL DEFAULT 0,
                `ServerId` BIGINT,
                `CreatedAt` DATETIME(6),
                `LastModifiedAt` DATETIME(6),
                FOREIGN KEY (`ServerId`) REFERENCES Servers(`ServerId`),
                PRIMARY KEY(`UserId`)
            );
            """)
        )

    def downgrade(self):
        self._cursor.execute('DROP TABLE `Servers`;')
        self._cursor.execute('DROP TABLE `Users`;')
